<?php

namespace App\Console\Commands\TencentCloud;

use Illuminate\Console\Command;
use TencentCloud\Common\Credential;
use TencentCloud\Common\Profile\ClientProfile;
use TencentCloud\Common\Profile\HttpProfile;
use TencentCloud\Common\Exception\TencentCloudSDKException;
use TencentCloud\Gme\V20180711\GmeClient;
use TencentCloud\Gme\V20180711\Models\DescribeScanResultListRequest;
use TencentCloud\Gme\V20180711\Models\ScanVoiceRequest;

class ScanVoiceCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'tcloud:voice';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        try {
            $credential = new Credential(config('tencentcloud.secretId'), config('tencentcloud.secretKey'));
            $httpProfile = new HttpProfile();
            $httpProfile->setEndpoint("gme.tencentcloudapi.com");
            $clientProfile = new ClientProfile();
            $clientProfile->setHttpProfile($httpProfile);
            $client = new GmeClient($credential, config('tencentcloud.region'), $clientProfile);


            $voiceList = $this->getVoiceList();
//            $voiceList = array_splice($voiceList,0,1);

//            $params = json_encode([
//                'BizId'=>1400260758,
//                'Scenes'=>['default'],
//                'Live'=>false,
//                'Callback'=>'https://msy.pocketuniversity.cn/api/callback',
//                'Tasks'=>$voiceList
//            ]);
//            $request = new ScanVoiceRequest();
//            $request->fromJsonString($params);
//            $response = $client->ScanVoice($request);
//            $response = json_decode($response->toJsonString(),true);

            $response = $this->getTaskList();


            $taskIdList = array_column($response['Data'], 'TaskId');

            $params = json_encode([
                'BizId' => 1400260758,
                'TaskIdList' => $taskIdList
            ]);
            $request = new DescribeScanResultListRequest();
            $request->fromJsonString($params);
            $response = $client->DescribeScanResultList($request);
            print_r($response->toJsonString());
        } catch (TencentCloudSDKException $e) {
            echo $e;
        }
    }


    public function getVoiceList()
    {
        return json_decode('[{
            "DataId":"128219",
            "Url":"https://oss.pocketuniversity.cn/media/2019-09-27/5d8dc1340c7fd.mp3"
        }]', true);
        return json_decode('[{
                    "DataId": "737096",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-28\/5d8e90cf31c3f.mp3?duration=8640"
                }, {
                    "DataId": "737095",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-28\/5d8e6d03ce2b2.mp3?duration=12096"
                }, {
                    "DataId": "737052",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-28\/5d8e3eeb20793.mp3?duration=4320"
                }, {
                    "DataId": "737002",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-28\/5d8e347f59151.mp3?duration=3960"
                }, {
                    "DataId": "736973",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e287ddc5ba.mp3?duration=3096"
                }, {
                    "DataId": "736971",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e268e5ebc5.mp3?duration=8352"
                }, {
                    "DataId": "736963",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-28\/5d8eafd516423.mp3?duration=20160"
                }, {
                    "DataId": "736920",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e251c9eedf.mp3?duration=18720"
                }, {
                    "DataId": "736799",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e0118d7662.mp3?duration=2664"
                }, {
                    "DataId": "736778",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e021d8bcfe.mp3?duration=5256"
                }, {
                    "DataId": "736766",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-28\/5d8ea0e235749.mp3?duration=4608"
                }, {
                    "DataId": "736607",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8da5c0653df.mp3?duration=8927"
                }, {
                    "DataId": "736601",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e12f1b6495.mp3?duration=4752"
                }, {
                    "DataId": "736572",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d9c3991802.mp3?duration=8928"
                }, {
                    "DataId": "736520",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d82d03c5dc.mp3?duration=6912"
                }, {
                    "DataId": "736519",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d98cd3c61c.mp3?duration=7992"
                }, {
                    "DataId": "736449",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d59ff0e5f3.mp3?duration=4464"
                }, {
                    "DataId": "736379",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8cf126cd34d.mp3?duration=5760"
                }, {
                    "DataId": "736376",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8cf11acc888.mp3?duration=15768"
                }, {
                    "DataId": "736371",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d555f7da2e.mp3?duration=5976"
                }, {
                    "DataId": "736342",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8cf0f2deef7.mp3?duration=12600"
                }, {
                    "DataId": "736315",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d97df9dee4.mp3?duration=6696"
                }, {
                    "DataId": "736273",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8dd82d8a75f.mp3?duration=7056"
                }, {
                    "DataId": "736246",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8dd47621a8c.mp3?duration=19944"
                }, {
                    "DataId": "736174",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cda81c6002.mp3?duration=2808"
                }, {
                    "DataId": "736173",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8ccc1216349.mp3?duration=14760"
                }, {
                    "DataId": "736163",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cd4115d861.mp3?duration=14616"
                }, {
                    "DataId": "736104",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cc70fc0f33.mp3?duration=20160"
                }, {
                    "DataId": "736080",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8dd8cb1a6a8.mp3?duration=14040"
                }, {
                    "DataId": "735939",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cbb47a174b.mp3?duration=4032"
                }, {
                    "DataId": "735934",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8ca1e3db7d7.mp3?duration=15480"
                }, {
                    "DataId": "735887",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cbcc5ac04f.mp3?duration=7488"
                }, {
                    "DataId": "735747",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c8afc94ce3.mp3?duration=11376"
                }, {
                    "DataId": "735740",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c6f60b676c.mp3?duration=20088"
                }, {
                    "DataId": "735680",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c95b6d3435.mp3?duration=15120"
                }, {
                    "DataId": "735674",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c641cebf94.mp3?duration=14976"
                }, {
                    "DataId": "735563",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c58392c1bb.mp3?duration=4032"
                }, {
                    "DataId": "735515",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c588452a54.mp3?duration=10080"
                }, {
                    "DataId": "735511",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c43907fa5c.mp3?duration=3456"
                }, {
                    "DataId": "735460",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c379416247.mp3?duration=6696"
                }, {
                    "DataId": "735416",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c32feb8b55.mp3?duration=17712"
                }, {
                    "DataId": "735381",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c3b5abc4cc.mp3?duration=3888"
                }, {
                    "DataId": "735301",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d6395a85bd.mp3?duration=5040"
                }, {
                    "DataId": "735262",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c261175e23.mp3?duration=6552"
                }, {
                    "DataId": "735214",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8ddf1f1e039.m4a?duration=32871"
                }, {
                    "DataId": "735154",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c10ce734c3.mp3?duration=6264"
                }, {
                    "DataId": "735128",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-24\/5d89fa42d6cea.mp3?duration=20160"
                }, {
                    "DataId": "735066",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c0bcbafb29.mp3?duration=8712"
                }, {
                    "DataId": "735012",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c3e75d15bb.mp3?duration=18000"
                }, {
                    "DataId": "735009",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cd53d5ef37.mp3?duration=3600"
                }, {
                    "DataId": "734949",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c5ee946919.mp3?duration=7848"
                }, {
                    "DataId": "734881",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8bb3ec2eb13.mp3?duration=13320"
                }, {
                    "DataId": "734804",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c4a5e4d711.mp3?duration=20160"
                }, {
                    "DataId": "734721",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cd8747f27a.mp3?duration=6912"
                }, {
                    "DataId": "734695",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8ba8ad016fd.mp3?duration=5904"
                }, {
                    "DataId": "734692",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8b9c1fa12d0.mp3?duration=20160"
                }, {
                    "DataId": "734662",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8b954e08f31.mp3?duration=5184"
                }, {
                    "DataId": "734332",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b86fa7d986.mp3?duration=18432"
                }, {
                    "DataId": "734234",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8dad1e2465a.mp3?duration=5472"
                }, {
                    "DataId": "734212",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8e25e2b5370.mp3?duration=3672"
                }, {
                    "DataId": "734085",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cd35918db7.mp3?duration=10872"
                }, {
                    "DataId": "734005",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8ddf999898f.mp3?duration=16056.000000000002"
                }, {
                    "DataId": "733996",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d968d6dec7.mp3?duration=3312"
                }, {
                    "DataId": "733825",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b899a6ce6e.mp3?duration=4608"
                }, {
                    "DataId": "733752",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8b9638ccf65.mp3?duration=14688"
                }, {
                    "DataId": "733681",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b761e5c888.mp3?duration=20160"
                }, {
                    "DataId": "733668",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8b945019591.mp3?duration=8496"
                }, {
                    "DataId": "733647",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d677233161.mp3?duration=6120"
                }, {
                    "DataId": "733630",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b8286a5b49.mp3?duration=5616"
                }, {
                    "DataId": "733629",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c144d2ff78.mp3?duration=3816"
                }, {
                    "DataId": "733620",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8cda781cddc.mp3?duration=6192"
                }, {
                    "DataId": "733594",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b71a38fa0d.mp3?duration=6120"
                }, {
                    "DataId": "733575",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b7a12ec25e.mp3?duration=7200"
                }, {
                    "DataId": "733481",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b7ecec1097.mp3?duration=3384"
                }, {
                    "DataId": "733473",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8ca133159b3.mp3?duration=15192"
                }, {
                    "DataId": "733243",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b6e1fb2b46.mp3?duration=5904"
                }, {
                    "DataId": "733208",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b65f037d32.mp3?duration=7992"
                }, {
                    "DataId": "733115",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b6a41dac6e.mp3?duration=11880"
                }, {
                    "DataId": "733096",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c159360a79.mp3?duration=16344.000000000002"
                }, {
                    "DataId": "733001",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8b9b7bd116d.mp3?duration=15192"
                }, {
                    "DataId": "732996",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b67a545518.mp3?duration=16056.000000000002"
                }, {
                    "DataId": "732980",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b71174c9c7.mp3?duration=9072"
                }, {
                    "DataId": "732973",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b68bb09b5a.mp3?duration=13320"
                }, {
                    "DataId": "732911",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b65020136f.mp3?duration=5472"
                }, {
                    "DataId": "732892",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b6ca955b2b.mp3?duration=10656"
                }, {
                    "DataId": "732870",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b6df33847c.mp3?duration=16200"
                }, {
                    "DataId": "732794",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b5e12c9162.mp3?duration=19584"
                }, {
                    "DataId": "732694",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8ca1e14df91.mp3?duration=4248"
                }, {
                    "DataId": "732692",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b84afd6b77.mp3?duration=2088"
                }, {
                    "DataId": "732683",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-27\/5d8d744b7611d.mp3?duration=19800"
                }, {
                    "DataId": "732646",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b6de257057.mp3?duration=20304"
                }, {
                    "DataId": "732588",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b791638ee5.mp3?duration=15480"
                }, {
                    "DataId": "732551",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b7359a00a6.mp3?duration=7200"
                }, {
                    "DataId": "732530",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b513f5f365.mp3?duration=17136"
                }, {
                    "DataId": "732516",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-26\/5d8c4251ae265.mp3?duration=7128"
                }, {
                    "DataId": "732503",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b4cae78804.mp3?duration=9864"
                }, {
                    "DataId": "732471",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b4c881aeb0.mp3?duration=12024"
                }, {
                    "DataId": "732436",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b3d9d51999.mp3?duration=4824"
                }, {
                    "DataId": "732425",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b2df77a2d2.mp3?duration=4032"
                }, {
                    "DataId": "732402",
                    "Url": "https:\/\/oss.pocketuniversity.cn\/media\/2019-09-25\/5d8b28bcd7cd9.mp3?duration=9360"
                }]', true);
    }


    public function getTaskList()
    {
        return json_decode('{
            "Data":[
                {
                    "DataId":"128219",
                    "TaskId":"2f827608-97cb-4d03-8b0c-056ba5ea0b21"
                }
            ]
        }', true);
        return json_decode('{
                    "Data": [{
                        "DataId": "737096",
                        "TaskId": "8c8530c2-d12e-4dff-ad09-5e90e391647a"
                    }, {
                        "DataId": "737095",
                        "TaskId": "3e0f4c2a-a6f2-4301-9426-a8364ebdf1d1"
                    }, {
                        "DataId": "737052",
                        "TaskId": "e47fb42b-0291-4ee2-9a7c-9e77a2ef5376"
                    }, {
                        "DataId": "737002",
                        "TaskId": "23a7dd80-af3f-4bfe-801e-af68b682b0d9"
                    }, {
                        "DataId": "736973",
                        "TaskId": "aaf0fa1b-aee2-46e8-ad63-ba8d1e7fc9e1"
                    }, {
                        "DataId": "736971",
                        "TaskId": "2545507d-6208-4835-87d4-0f40747dd224"
                    }, {
                        "DataId": "736963",
                        "TaskId": "7488ef93-5749-4516-a0ea-d1063c1b691b"
                    }, {
                        "DataId": "736920",
                        "TaskId": "113998b0-2011-4f13-b769-2b72ea20d1e5"
                    }, {
                        "DataId": "736799",
                        "TaskId": "0686e6c1-a409-437d-9195-4748aea92dad"
                    }, {
                        "DataId": "736778",
                        "TaskId": "84dcf4fe-b753-49b5-b112-ae5b5cc81ff2"
                    }, {
                        "DataId": "736766",
                        "TaskId": "c29f1fd1-f86e-4aa9-b036-6c9259d35b75"
                    }, {
                        "DataId": "736607",
                        "TaskId": "4ed7b36d-569c-44bd-9fa6-075d923ebe64"
                    }, {
                        "DataId": "736601",
                        "TaskId": "6b34354f-43ef-4603-957c-4e77a7c102ea"
                    }, {
                        "DataId": "736572",
                        "TaskId": "0e57553a-3fa3-41fa-a158-77f5a734e99a"
                    }, {
                        "DataId": "736520",
                        "TaskId": "b5b020f0-ef35-4ce5-8ad3-3c709740426a"
                    }, {
                        "DataId": "736519",
                        "TaskId": "d52f6f69-411f-4db4-adaf-961dd1b703d4"
                    }, {
                        "DataId": "736449",
                        "TaskId": "99c6e3be-58ce-4f2e-9f1d-62f06edb6f4d"
                    }, {
                        "DataId": "736379",
                        "TaskId": "a70106d4-77c9-49e2-b5f5-c0c5481276f4"
                    }, {
                        "DataId": "736376",
                        "TaskId": "9f4fc265-1e57-4325-931b-03fb21d5f744"
                    }, {
                        "DataId": "736371",
                        "TaskId": "ff5baccc-a924-448e-8756-87cd38511360"
                    }, {
                        "DataId": "736342",
                        "TaskId": "b4cfd5f2-1e11-4ccf-9833-7843d770fba7"
                    }, {
                        "DataId": "736315",
                        "TaskId": "cb6b0b5b-c7e3-4d54-b766-b66a62f2c8bb"
                    }, {
                        "DataId": "736273",
                        "TaskId": "7f3925f5-2c23-46c3-95dc-7ea3d0e06240"
                    }, {
                        "DataId": "736246",
                        "TaskId": "5586880a-55a9-4818-b215-debb701011a3"
                    }, {
                        "DataId": "736174",
                        "TaskId": "309ab371-28e4-4cc3-baf9-3381ca33f005"
                    }, {
                        "DataId": "736173",
                        "TaskId": "535ca824-ad13-4c53-b824-aa204ed1e876"
                    }, {
                        "DataId": "736163",
                        "TaskId": "36cdbb16-2333-44c3-8cae-31eff67a6d88"
                    }, {
                        "DataId": "736104",
                        "TaskId": "c53fcd53-208c-448b-b03c-6b14a31993de"
                    }, {
                        "DataId": "736080",
                        "TaskId": "86893938-1dcd-40d0-bec2-38d9f9ff15d4"
                    }, {
                        "DataId": "735939",
                        "TaskId": "bc649946-6b91-4ea0-adef-b493d993e540"
                    }, {
                        "DataId": "735934",
                        "TaskId": "64b5ebb1-5ee8-43df-ad4b-47c05f081e5c"
                    }, {
                        "DataId": "735887",
                        "TaskId": "9b4c5e59-0b1f-4cd1-ba19-d8f3c7fa629f"
                    }, {
                        "DataId": "735747",
                        "TaskId": "a0ad9c26-9d1b-4740-ad0e-566ba4991506"
                    }, {
                        "DataId": "735740",
                        "TaskId": "f93e9a1a-28c7-4e21-9cca-b46572b387a3"
                    }, {
                        "DataId": "735680",
                        "TaskId": "12b14ba2-f357-4b42-9b7f-7ce847d4f3c8"
                    }, {
                        "DataId": "735674",
                        "TaskId": "48f0a8e6-ce12-4f67-a681-23d6d5219dad"
                    }, {
                        "DataId": "735563",
                        "TaskId": "0c3b619f-663c-4d15-b528-d1e1dbed600f"
                    }, {
                        "DataId": "735515",
                        "TaskId": "bd8cd4e2-2578-4c20-8e1f-f05dbc741594"
                    }, {
                        "DataId": "735511",
                        "TaskId": "9e723a66-0bd8-4d25-b1d6-3a4cf5fdac5d"
                    }, {
                        "DataId": "735460",
                        "TaskId": "d2fe8c0e-2be6-48aa-b4da-0cf8edbaffd5"
                    }, {
                        "DataId": "735416",
                        "TaskId": "a5360782-f977-4db0-802c-59864c85a645"
                    }, {
                        "DataId": "735381",
                        "TaskId": "1b457365-27d6-429b-8366-ca253168dbdd"
                    }, {
                        "DataId": "735301",
                        "TaskId": "05183887-33ab-471a-b9e3-3a5484ede2df"
                    }, {
                        "DataId": "735262",
                        "TaskId": "12f11a2b-71d2-477e-bb6a-b6c90b663114"
                    }, {
                        "DataId": "735214",
                        "TaskId": "f358f2f3-dbde-4cf6-b5b0-6d9b1b405452"
                    }, {
                        "DataId": "735154",
                        "TaskId": "c29f2d83-46d0-4219-8a36-4f160e5a8776"
                    }, {
                        "DataId": "735128",
                        "TaskId": "364d1bd6-7267-44fc-b4d3-99b7302b8322"
                    }, {
                        "DataId": "735066",
                        "TaskId": "ec9f9f28-5e62-423b-94fb-a29375c00a39"
                    }, {
                        "DataId": "735012",
                        "TaskId": "c7517ca2-7907-4543-9e8f-e45301968586"
                    }, {
                        "DataId": "735009",
                        "TaskId": "8b8678a5-6c80-4a45-9cda-4d52fa8838af"
                    }, {
                        "DataId": "734949",
                        "TaskId": "56e2b9c5-59ef-4db8-8b04-8596cc592af5"
                    }, {
                        "DataId": "734881",
                        "TaskId": "a7185bb7-eaa7-47b5-91b7-5fa1a83e7c78"
                    }, {
                        "DataId": "734804",
                        "TaskId": "548a35c8-4956-465f-836b-b36b42affc2b"
                    }, {
                        "DataId": "734721",
                        "TaskId": "70e841f0-15af-4a20-b8e3-2d213b591165"
                    }, {
                        "DataId": "734695",
                        "TaskId": "df4b142e-3572-4a98-9617-eadb4a9dd164"
                    }, {
                        "DataId": "734692",
                        "TaskId": "c9991a0b-8006-4398-ad0f-71efc82f0544"
                    }, {
                        "DataId": "734662",
                        "TaskId": "33fb6dc3-6c98-4868-af34-e15b98fe18f0"
                    }, {
                        "DataId": "734332",
                        "TaskId": "98ed5069-49b5-4af0-bdf7-3794b2c6401b"
                    }, {
                        "DataId": "734234",
                        "TaskId": "b39d38fe-0ee6-4454-b84f-46ad8884e1ff"
                    }, {
                        "DataId": "734212",
                        "TaskId": "05f55f4e-bc73-43c1-a2c5-dc2d867b0d04"
                    }, {
                        "DataId": "734085",
                        "TaskId": "60bc6bf3-31b7-4dce-aa7a-2dd78bbcd803"
                    }, {
                        "DataId": "734005",
                        "TaskId": "a0b9af3e-ec19-4f60-861b-671b4c4c2a72"
                    }, {
                        "DataId": "733996",
                        "TaskId": "ece6bfbf-4549-4543-87f2-276f46416d44"
                    }, {
                        "DataId": "733825",
                        "TaskId": "76cfd925-eef7-44bc-93b6-e741cf86af0f"
                    }, {
                        "DataId": "733752",
                        "TaskId": "229e5e8e-a7c5-4450-9458-5b6e3051ac85"
                    }, {
                        "DataId": "733681",
                        "TaskId": "af89ab6b-c24f-43c6-abbc-013d6cf2e475"
                    }, {
                        "DataId": "733668",
                        "TaskId": "5f376898-14e9-44d1-8569-514dd45d385a"
                    }, {
                        "DataId": "733647",
                        "TaskId": "4fc02460-1cc2-46e0-9726-5e814a01ca0a"
                    }, {
                        "DataId": "733630",
                        "TaskId": "ff4f03f7-bf75-454b-a299-79f3cf108dec"
                    }, {
                        "DataId": "733629",
                        "TaskId": "2ab3d15d-a759-4be2-bc3a-7755bf9cf50a"
                    }, {
                        "DataId": "733620",
                        "TaskId": "608a47ea-b91e-4365-b5a9-c05a933c2e79"
                    }, {
                        "DataId": "733594",
                        "TaskId": "73adf3f7-1748-481d-9b4b-ffe06a46ccef"
                    }, {
                        "DataId": "733575",
                        "TaskId": "8eb4ab45-2a8e-4988-874e-e5de88c68d57"
                    }, {
                        "DataId": "733481",
                        "TaskId": "62e36a6c-3f78-421b-8578-b81463ced68b"
                    }, {
                        "DataId": "733473",
                        "TaskId": "762fe4fe-dc65-4537-81f1-0e3a6b1ebbb7"
                    }, {
                        "DataId": "733243",
                        "TaskId": "bace2f25-f44e-4ec9-9b97-aae3bfe12f27"
                    }, {
                        "DataId": "733208",
                        "TaskId": "8d03a083-ff32-40cc-a488-2e075fbb5df7"
                    }, {
                        "DataId": "733115",
                        "TaskId": "8559beeb-3728-4558-911b-4ee8199f34b8"
                    }, {
                        "DataId": "733096",
                        "TaskId": "ed68c196-e8c9-46ba-be0a-7191bf475354"
                    }, {
                        "DataId": "733001",
                        "TaskId": "16cb807a-4425-427f-b400-919a1d39ac61"
                    }, {
                        "DataId": "732996",
                        "TaskId": "a4c3fa5f-7915-4ff5-85fb-e0ac935b18a8"
                    }, {
                        "DataId": "732980",
                        "TaskId": "a0b050e5-7422-48d1-aad1-eeba0f9b0dbc"
                    }, {
                        "DataId": "732973",
                        "TaskId": "12f790d0-37b1-4fcd-bf11-ee4170c32800"
                    }, {
                        "DataId": "732911",
                        "TaskId": "c0cdb4a2-5893-48ef-8278-0b2637697a9a"
                    }, {
                        "DataId": "732892",
                        "TaskId": "3cfbf55d-aec4-4493-8794-d77159bbd47a"
                    }, {
                        "DataId": "732870",
                        "TaskId": "72e88bab-85d9-4293-8655-000260fbe726"
                    }, {
                        "DataId": "732794",
                        "TaskId": "3a755cda-6b8f-4f44-9a91-96ef17f93dbb"
                    }, {
                        "DataId": "732694",
                        "TaskId": "94ac3dca-56c4-4888-80fe-917d676a22d2"
                    }, {
                        "DataId": "732692",
                        "TaskId": "ece1472d-f037-4c4c-ae6b-092de590d795"
                    }, {
                        "DataId": "732683",
                        "TaskId": "52d7ce21-fb36-47ff-9932-e5994abb1559"
                    }, {
                        "DataId": "732646",
                        "TaskId": "e57c08f2-913d-4eed-b393-c4cd96e2a392"
                    }, {
                        "DataId": "732588",
                        "TaskId": "d0fc9e1e-e227-4158-98f6-2c4cba935d1c"
                    }, {
                        "DataId": "732551",
                        "TaskId": "c8913a19-c9c8-44a0-bf70-2ee404a6a550"
                    }, {
                        "DataId": "732530",
                        "TaskId": "aec8c070-4aa7-4864-88cd-4e136c962bca"
                    }, {
                        "DataId": "732516",
                        "TaskId": "0b9bc55e-b2bf-404a-ac0b-ecd70e5b7cad"
                    }, {
                        "DataId": "732503",
                        "TaskId": "27067305-e5e5-417f-9a30-ca09f05b428c"
                    }, {
                        "DataId": "732471",
                        "TaskId": "993019fd-1f84-4c67-942e-1079ee1d00af"
                    }, {
                        "DataId": "732436",
                        "TaskId": "56a5dcb1-6bec-4fdf-a6c6-5cb262c7585b"
                    }, {
                        "DataId": "732425",
                        "TaskId": "c83d511a-d264-4222-8a5b-3f26f31fe89e"
                    }, {
                        "DataId": "732402",
                        "TaskId": "cc060023-069e-44a0-8e9d-541ac78ec73d"
                    }],
                    "RequestId": "b04e71c3-1704-4181-a8aa-0646953de752"
                }', true);
    }
}
