<?php

namespace App\Providers;

use App\Events\GoodnightSuccess;
use App\Listeners\CompleteInfoListen;
use App\Listeners\FriendMapChange;
use Illuminate\Foundation\Support\Providers\EventServiceProvider as ServiceProvider;

/**
 * Class EventServiceProvider
 * @package App\Providers
 */
class EventServiceProvider extends ServiceProvider
{
    /**
     * The event listener mappings for the application.
     *
     * @var array
     */
    protected $listen = [
        'App\Events\CreatedPartner' => [
            'App\Listeners\MockThumb',
            "App\Listeners\ReportPartnerChange@createdPartner",
            'App\Listeners\UserAction@createdPartner',
        ],
        'App\Events\UpdatedPartner' => [
            'App\Listeners\ReportPartnerChange@updatedPartner',
            'App\Listeners\UserAction@updatedPartnerState'
        ],
        'App\Events\HidePartner' => [
            'App\Listeners\ReportPartnerChange@hidePartner'
        ],
        'App\Events\AuditMedia' => [
            'App\Listeners\AuditMedia'
        ],
        'App\Events\AuditMediaDeal' => [
            'App\Listeners\AuditMediaDeal'
        ],
        'App\Events\PartnerFeedPoolChange' => [
            'App\Listeners\PartnerFeedPoolChange'
        ],
        'App\Events\UpdatedUserInfo' => [
            'App\Listeners\UserAction@updatedUserInfo'
        ],
        'App\Events\LogoutUser' => [
            'App\Listeners\UserAction@logoutUser'
        ],
        'App\Events\BlackUser' => [
            'App\Listeners\UserAction@blackUser'
        ],
        'App\Events\CheckPartner' => [
            'App\Listeners\UserAction@checkPartner'
        ],
        'App\Events\EachLike' => [
            'App\Listeners\EachLikeNotice',
            'App\Listeners\EachLikeReport',
             'App\Listeners\FriendMapChange@eachLike',
        ],
        'App\Events\UnEachLike' => [
             'App\Listeners\FriendMapChange@unEachLike'
        ],
        'App\Events\PairSuccess' => [
             'App\Listeners\FriendMapChange@pairSuccess'
        ],
        'App\Events\GoodnightSuccess' => [
            'App\Listeners\FriendMapChange@goodnight'
        ],
        'App\Events\ImSendMsg' => [
            'App\Listeners\UpdateContactListCache'
        ],
        'App\Events\ThumbUser' => [
            'App\Listeners\ThumbOrUnThumbUser@thumb'
        ],
        'App\Events\UnThumbUser' => [
            'App\Listeners\ThumbOrUnThumbUser@unThumb'
        ]
    ];

    protected $subscribe = [
        CompleteInfoListen::class,
    ];

    /**
     * Register any events for your application.
     *
     * @return void
     */
    public function boot()
    {
        parent::boot();

        //
    }
}
